@@ -18,7 +18,7 @@
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2007.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
@@ -41,9 +41,10 @@
 #include <string.h>
 
 #include "gdkinputprivate.h"
-
+#include "gdksettings.c"
 #include "gdkalias.h"
 
+
 #ifdef HAVE_XKB
 #include <X11/XKBlib.h>
 #endif
@@ -110,7 +111,7 @@ static GdkFilterReturn gdk_wm_protocols_filter (GdkXEvent *xev,
 static GSource *gdk_display_source_new (GdkDisplay *display);
 static gboolean gdk_check_xpending     (GdkDisplay *display);
 
-static void gdk_xsettings_watch_cb  (Window            window,
+static Bool gdk_xsettings_watch_cb  (Window            window,
 				     Bool              is_start,
 				     long              mask,
 				     void             *cb_data);
@@ -176,15 +177,13 @@ _gdk_x11_events_init_screen (GdkScreen *screen)
   /* Keep a flag to avoid extra notifies that we don't need
    */
   screen_x11->xsettings_in_init = TRUE;
-  screen_x11->xsettings_client = xsettings_client_new (screen_x11->xdisplay,
-						       screen_x11->screen_num,
-						       gdk_xsettings_notify_cb,
-						       gdk_xsettings_watch_cb,
-						       screen);
-  xsettings_client_set_grab_func (screen_x11->xsettings_client,
-				  refcounted_grab_server);
-  xsettings_client_set_ungrab_func (screen_x11->xsettings_client,
-				    refcounted_ungrab_server);
+  screen_x11->xsettings_client = xsettings_client_new_with_grab_funcs (screen_x11->xdisplay,
+						                       screen_x11->screen_num,
+						                       gdk_xsettings_notify_cb,
+						                       gdk_xsettings_watch_cb,
+						                       screen,
+                                                                       refcounted_grab_server,
+                                                                       refcounted_ungrab_server);
   screen_x11->xsettings_in_init = FALSE;
 }
 
@@ -360,6 +359,8 @@ gdk_event_apply_filters (XEvent *xevent,
  * @data: user data to pass to @func.
  *
  * Adds a filter to be called when X ClientMessage events are received.
+ * See gdk_window_add_filter() if you are interested in filtering other
+ * types of events.
  *
  * Since: 2.2
  **/ 
@@ -2100,6 +2101,34 @@ gdk_event_translate (GdkDisplay *display,
 	}
       else
 #endif
+#if defined(HAVE_XCOMPOSITE) && defined (HAVE_XDAMAGE) && defined (HAVE_XFIXES)
+      if (display_x11->have_xdamage && window_private->composited &&
+	  xevent->type == display_x11->xdamage_event_base + XDamageNotify)
+	{
+	  XDamageNotifyEvent *damage_event = (XDamageNotifyEvent *) xevent;
+	  XserverRegion repair;
+	  GdkRectangle rect;
+
+	  rect.x = window_private->x + damage_event->area.x;
+	  rect.y = window_private->y + damage_event->area.y;
+	  rect.width = damage_event->area.width;
+	  rect.height = damage_event->area.height;
+
+	  repair = XFixesCreateRegion (display_x11->xdisplay,
+				      &damage_event->area, 1);
+	  XDamageSubtract (display_x11->xdisplay,
+			   window_impl->damage,
+			   repair, None);
+	  XFixesDestroyRegion (display_x11->xdisplay, repair);
+
+	  if (window_private->parent != NULL)
+	    _gdk_window_process_expose (GDK_WINDOW (window_private->parent),
+					damage_event->serial, &rect);
+
+	  return_val = TRUE;
+	}
+      else
+#endif
 	{
 	  /* something else - (e.g., a Xinput event) */
 	  
@@ -2192,13 +2221,13 @@ gdk_wm_protocols_filter (GdkXEvent *xev,
 	   !_gdk_x11_display_is_root_window (display,
 					     xevent->xclient.window))
     {
-      XEvent xev = *xevent;
+      XClientMessageEvent xclient = xevent->xclient;
       
-      xev.xclient.window = GDK_WINDOW_XROOTWIN (win);
+      xclient.window = GDK_WINDOW_XROOTWIN (win);
       XSendEvent (GDK_WINDOW_XDISPLAY (win), 
-		  xev.xclient.window,
+		  xclient.window,
 		  False, 
-		  SubstructureRedirectMask | SubstructureNotifyMask, &xev);
+		  SubstructureRedirectMask | SubstructureNotifyMask, (XEvent *)&xclient);
 
       return GDK_FILTER_REMOVE;
     }
@@ -2811,7 +2840,6 @@ gdk_net_wm_supports (GdkAtom property)
   return gdk_x11_screen_supports_net_wm_hint (gdk_screen_get_default (), property);
 }
 
-#include "gdksettings.c"
 
 static void
 gdk_xsettings_notify_cb (const char       *name,
@@ -2993,7 +3021,7 @@ gdk_xsettings_client_event_filter (GdkXEvent *xevent,
     return GDK_FILTER_CONTINUE;
 }
 
-static void 
+static Bool
 gdk_xsettings_watch_cb (Window   window,
 			Bool	 is_start,
 			long     mask,
@@ -3006,19 +3034,39 @@ gdk_xsettings_watch_cb (Window   window,
 
   if (is_start)
     {
-      if (!gdkwin)
-	gdkwin = gdk_window_foreign_new_for_display (gdk_screen_get_display (screen), window);
-      else
+      if (gdkwin)
 	g_object_ref (gdkwin);
-      
+      else
+	{
+	  gdkwin = gdk_window_foreign_new_for_display (gdk_screen_get_display (screen), window);
+	  
+	  /* gdk_window_foreign_new_for_display() can fail and return NULL if the
+	   * window has already been destroyed.
+	   */
+	  if (!gdkwin)
+	    return False;
+	}
+
       gdk_window_add_filter (gdkwin, gdk_xsettings_client_event_filter, screen);
     }
   else
     {
-      g_assert (gdkwin);
+      if (!gdkwin)
+	{
+	  /* gdkwin should not be NULL here, since if starting the watch succeeded
+	   * we have a reference on the window. It might mean that the caller didn't
+	   * remove the watch when it got a DestroyNotify event. Or maybe the
+	   * caller ignored the return value when starting the watch failed.
+	   */
+	  g_warning ("gdk_xsettings_watch_cb(): Couldn't find window to unwatch");
+	  return False;
+	}
+      
       gdk_window_remove_filter (gdkwin, gdk_xsettings_client_event_filter, screen);
       g_object_unref (gdkwin);
     }
+
+  return True;
 }
 
 #define __GDK_EVENTS_X11_C__
