@@ -24,8 +24,8 @@
 #include "gtkicontheme.h"
 #include "gtkmodules.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
 #include "gtkstock.h"
+#include "gtkalias.h"
 
 #include <string.h>
 
@@ -103,14 +103,10 @@ gtk_file_info_free (GtkFileInfo *info)
 {
   g_return_if_fail (info != NULL);
 
-  if (info->display_name)
-    g_free (info->display_name);
-  if (info->mime_type)
-    g_free (info->mime_type);
-  if (info->display_key)
-    g_free (info->display_key);
-  if (info->icon_name)
-    g_free (info->icon_name);
+  g_free (info->display_name);
+  g_free (info->mime_type);
+  g_free (info->display_key);
+  g_free (info->icon_name);
 
   g_free (info);
 }
@@ -158,8 +154,7 @@ gtk_file_info_set_display_name (GtkFileInfo *info,
   if (display_name == info->display_name)
     return;
 
-  if (info->display_name)
-    g_free (info->display_name);
+  g_free (info->display_name);
   if (info->display_key)
     {
       g_free (info->display_key);
@@ -217,8 +212,7 @@ gtk_file_info_set_mime_type (GtkFileInfo *info,
 {
   g_return_if_fail (info != NULL);
   
-  if (info->mime_type)
-    g_free (info->mime_type);
+  g_free (info->mime_type);
 
   info->mime_type = g_strdup (mime_type);
 }
@@ -264,8 +258,7 @@ gtk_file_info_set_icon_name (GtkFileInfo *info,
 {
   g_return_if_fail (info != NULL);
   
-  if (info->icon_name)
-    g_free (info->icon_name);
+  g_free (info->icon_name);
 
   info->icon_name = g_strdup (icon_name);
 }
@@ -301,15 +294,20 @@ gtk_file_info_render_icon (const GtkFileInfo  *info,
           GtkIconTheme *icon_theme;
 
 	  icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
-	  pixbuf = gtk_icon_theme_load_icon (icon_theme, info->icon_name,
-					     pixel_size, 0, NULL);
+          if (gtk_icon_theme_has_icon (icon_theme, info->icon_name))
+            pixbuf = gtk_icon_theme_load_icon (icon_theme, info->icon_name,
+                                               pixel_size, 0, NULL);
 	}
     }
 
   if (!pixbuf)
     {
       /* load a fallback icon */
-      pixbuf = gtk_widget_render_icon (widget, GTK_STOCK_FILE, GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
+      pixbuf = gtk_widget_render_icon (widget,
+                                       gtk_file_info_get_is_folder (info)
+                                        ? GTK_STOCK_DIRECTORY : GTK_STOCK_FILE,
+                                       GTK_ICON_SIZE_SMALL_TOOLBAR,
+                                       NULL);
       if (!pixbuf && error)
         g_set_error (error,
 		     GTK_FILE_SYSTEM_ERROR,
@@ -397,7 +395,7 @@ gtk_file_system_get_type (void)
 
   if (!file_system_type)
     {
-      static const GTypeInfo file_system_info =
+      const GTypeInfo file_system_info =
       {
 	sizeof (GtkFileSystemIface),  /* class_size */
 	gtk_file_system_base_init,    /* base_init */
@@ -496,7 +494,7 @@ gtk_file_system_cancel_operation (GtkFileSystemHandle *handle)
 {
   g_return_if_fail (GTK_IS_FILE_SYSTEM_HANDLE (handle));
 
-  return GTK_FILE_SYSTEM_GET_IFACE (handle->file_system)->cancel_operation (handle);
+  GTK_FILE_SYSTEM_GET_IFACE (handle->file_system)->cancel_operation (handle);
 }
 
 /**
@@ -648,7 +646,7 @@ gtk_file_system_volume_render_icon (GtkFileSystem        *file_system,
 				    GError              **error)
 {
   gchar *icon_name;
-  GdkPixbuf *pixbuf;
+  GdkPixbuf *pixbuf = NULL;
 
   g_return_val_if_fail (GTK_IS_FILE_SYSTEM (file_system), NULL);
   g_return_val_if_fail (volume != NULL, NULL);
@@ -658,14 +656,22 @@ gtk_file_system_volume_render_icon (GtkFileSystem        *file_system,
 
   icon_name = gtk_file_system_volume_get_icon_name (file_system, volume,
 						    error);
-  if (!icon_name)
+  if (icon_name)
     {
-      return NULL;
+      GtkIconTheme *icon_theme;
+
+      icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
+      if (gtk_icon_theme_has_icon (icon_theme, icon_name))
+        pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                           icon_name, pixel_size, 0, NULL);
+      g_free (icon_name);
     }
 
-  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget)),
-				     icon_name, pixel_size, 0, NULL);
-  g_free (icon_name);
+  if (!pixbuf)
+    pixbuf = gtk_widget_render_icon (widget,
+                                     GTK_STOCK_HARDDISK,
+                                     GTK_ICON_SIZE_SMALL_TOOLBAR,
+                                     NULL);
 
   return pixbuf;
 }
@@ -1018,7 +1024,7 @@ gtk_file_folder_get_type (void)
 
   if (!file_folder_type)
     {
-      static const GTypeInfo file_folder_info =
+      const GTypeInfo file_folder_info =
       {
 	sizeof (GtkFileFolderIface),  /* class_size */
 	gtk_file_folder_base_init,    /* base_init */
@@ -1170,7 +1176,11 @@ gtk_file_path_get_type (void)
 GSList *
 gtk_file_paths_sort (GSList *paths)
 {
+#ifndef G_OS_WIN32
   return g_slist_sort (paths, (GCompareFunc)strcmp);
+#else
+  return g_slist_sort (paths, (GCompareFunc)_gtk_file_system_win32_path_compare);
+#endif
 }
 
 /**
@@ -1245,7 +1255,7 @@ struct _GtkFileSystemModuleClass
   GTypeModuleClass parent_class;
 };
 
-G_DEFINE_TYPE (GtkFileSystemModule, _gtk_file_system_module, G_TYPE_TYPE_MODULE);
+G_DEFINE_TYPE (GtkFileSystemModule, _gtk_file_system_module, G_TYPE_TYPE_MODULE)
 #define GTK_TYPE_FILE_SYSTEM_MODULE       (_gtk_file_system_module_get_type ())
 #define GTK_FILE_SYSTEM_MODULE(module)	  (G_TYPE_CHECK_INSTANCE_CAST ((module), GTK_TYPE_FILE_SYSTEM_MODULE, GtkFileSystemModule))
 
