@@ -2875,5 +2875,6 @@ cups_printer_get_capabilities (GtkPrinter *printer)
   return
     GTK_PRINT_CAPABILITY_COPIES |
     GTK_PRINT_CAPABILITY_COLLATE |
-    GTK_PRINT_CAPABILITY_REVERSE;
+    GTK_PRINT_CAPABILITY_REVERSE |
+    GTK_PRINT_CAPABILITY_NUMBER_UP;
 }
